@@ -17,11 +17,10 @@ KIND, either express or implied.  See the License for the
 specific language governing permissions and limitations
 under the License.
 """
-# Use json as first choice, simplejson as second choice.
 try:
-    import json
-except ImportError:
     import simplejson as json
+except ImportError:
+    import json
 
 from riak.bucket import RiakBucket
 from riak.mapreduce import RiakMapReduce
@@ -396,6 +395,9 @@ class RiakClient(object):
 
     def solr(self):
         if self._solr is None:
-            self._solr = RiakSearch(self, host=self._host, port=self._port)
+            self._solr = RiakSearch(self, 
+                                    host=self._host, 
+                                    port=self._port,
+                                    transport_class=type(self._transport))
 
         return self._solr
