@@ -27,9 +27,9 @@ import httplib
 import socket
 import errno
 try:
-    import json
-except ImportError:
     import simplejson as json
+except ImportError:
+    import json
 
 from transport import RiakTransport
 from riak.metadata import *
@@ -38,6 +38,7 @@ from riak import RiakError
 from riak.riak_index_entry import RiakIndexEntry
 from riak.multidict import MultiDict
 from connection import HTTPConnectionManager
+from connection import HTTPSConnectionManager
 import riak.util
 from xml.etree import ElementTree
 
@@ -682,6 +683,16 @@ class RiakHttpTransport(RiakTransport):
         return retVal
 
 
+class RiakHttpsTransport(RiakHttpTransport):
+    """
+    The RiakHttpsTransport object holds information necessary to
+    connect to a Riak cluster running with SSL enabled.
+    """
+
+    # The ConnectionManager class that this transport prefers.
+    default_cm = HTTPSConnectionManager
+
+
 class XMLSearchResult(object):
     # Match tags that are document fields
     fieldtags = ['str', 'int', 'date']
